#!/usr/bin/ruby
# encoding: utf-8

require 'plot'
require 'complex'

def fft(a, sign = 1)
  return a if a.size == 1
  even = fft(a[0...a.size/2], sign)
  odd = fft(a[a.size/2..-1], sign)

  n = a.size
  wn = Complex(Math.cos(2 * Math::PI / n), sign * Math.sin(2 * Math::PI / n))
  y = []
  w = Complex(1)
  (0...a.size/2).each do |j|
    y[j] = even[j] + w*odd[j]
    y[j + a.size/2] = even[j] - w*odd[j]
    w *= wn
  end
  y
end

def sort(a)
  return a if a.size == 1
  r = [[],[]]
  a.each_with_index { |p, i|  r[i%2] << p }
  sort(r[0]) + sort(r[1])
end

plot("cos(x) + sin(3*x)", :xrange => "[0:2*pi]", :filename => "images/ruby-source.eps")

N = 31
x, a = 0, []
step = 2*Math::PI / N
until x > 2*Math::PI
  a << Complex(Math.cos(x) + Math.sin(3*x))
  x += step
end

y1 = fft(sort(a))
data = [(0..N).to_a, y1.map{|i| i.abs}]
plot(data, :xrange => "[0:#{N}]", :filename => "images/ruby-fft.eps")
plot(data, :xrange => "[0:#{N}]", :filename => "images/ruby-fft-stem.eps", :style => "stem")

y2 = fft(sort(y1), -1)
data = [(0..N).to_a, y2.map{|i| i/y1.size}]
plot(data, :xrange => "[0:#{N}]", :filename => "images/ruby-ifft.eps")
plot(data, :xrange => "[0:#{N}]", :filename => "images/ruby-ifft-stem.eps", :style => "stem")
